package com.gx.obe.evaluation.composite;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.components.core.Constants;
import com.gx.obe.util.utils.ObjectUtils;

/**
 * @Description: 工程人材机表
 * @author guoyr
 */
public class MachineTableComposite extends Composite {

	private Grid grid;

	/**
	 * @param parent
	 * @param style
	 */
	public MachineTableComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		grid = new Grid(this, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		grid.setHeaderVisible(true);
		
		GridColumn gridColumn = new GridColumn(grid, SWT.NONE);
		gridColumn.setText("编码");
		gridColumn.setWidth(70);
		
		GridColumn gridColumn_1 = new GridColumn(grid, SWT.NONE);
		gridColumn_1.setText("材料名称");
		gridColumn_1.setWidth(150);
		gridColumn_1.setWordWrap(true);
		
		GridColumn gridColumn_2 = new GridColumn(grid, SWT.NONE);
		gridColumn_2.setText("规格型号");
		gridColumn_2.setWidth(100);
		gridColumn_2.setWordWrap(true);
		
		GridColumn gridColumn_3 = new GridColumn(grid, SWT.CENTER);
		gridColumn_3.setText("单位");
		gridColumn_3.setWidth(60);
		
		GridColumn gridColumn_4 = new GridColumn(grid, SWT.RIGHT);
		gridColumn_4.setText("数量");
		gridColumn_4.setWidth(100);
		
		GridColumn gridColumn_5 = new GridColumn(grid, SWT.RIGHT);
		gridColumn_5.setText("单价(元)");
		gridColumn_5.setWidth(100);
		
		GridColumn gridColumn_6 = new GridColumn(grid, SWT.RIGHT);
		gridColumn_6.setText("合价");
		gridColumn_6.setWidth(100);
		
		GridItem gridItem = new GridItem(grid, SWT.NONE);
		gridItem.setText("New Row");
		
		init();
	}
	
	/**
	 * @Description: 初始化界面数据
	 * @author guoyr
	 */
	private void init(){
		grid.setItemHeight(Constants.ROW_HIGHT);
		refreshGrid();
	}
	
	/**
	 * @Description:刷新Grid列表 
	 * @author guoyr
	 */
	private void refreshGrid(){
		grid.removeAll();
		
		List<Map<String, Object>> contentList = getContentList();
		if(null != contentList && contentList.size() > 0){
			for(Map<String, Object> contentMap : contentList){
				GridItem gridItem = new GridItem(grid, SWT.NONE);
				gridItem.setText(0, ObjectUtils.getObjString(contentMap.get("code")));
				gridItem.setText(1, ObjectUtils.getObjString(contentMap.get("materialName")));
				gridItem.setText(2, ObjectUtils.getObjString(contentMap.get("specificModel")));
				gridItem.setText(3, ObjectUtils.getObjString(contentMap.get("unit")));
				gridItem.setText(4, ObjectUtils.getObjString(contentMap.get("amount")));
				gridItem.setText(5, ObjectUtils.getObjString(contentMap.get("price")));
				gridItem.setText(6, ObjectUtils.getObjString(contentMap.get("sum")));
				gridItem.setData("contentMap", contentMap);
			}
		}
	}
	
	/**
	 * @Description: 获得Grid数据
	 * @author guoyr
	 * @return
	 */
	private List<Map<String, Object>> getContentList(){
		List<Map<String, Object>> contentList = new ArrayList<Map<String, Object>>();
		String code = "0233011";// 编码
		String materialName = "草袋";// 材料名称
		String specificModel = "综合";
		String unit = "个";// 单位
		double amount = 25;// 数量
		double price = 0.9017;// 单价
		double sum = 22.54;// 合价
		for(int i = 0; i < 3; i++){
			switch(i){
				case 0:
					code = "0351001";
					materialName = "圆钉";
					specificModel = "";
					unit = "kg";
					amount = 46.793907;
					price = 6.7628;
					sum = 316.46;
					break;
				case 1:
					code = "0401001";
					materialName = "水泥";
					specificModel = "32.5";
					unit = "t";
					amount = 134.442;
					price = 414.7809;
					sum = 55763.97;
					break;
				case 2:
					code = "0405001";
					materialName = "碎石";
					specificModel = "综合";
					unit = "t";
					amount = 1900.455589;
					price = 64.0205;
					sum = 121668.12;
					break;
			}
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("code", code);
			map.put("materialName", materialName);
			map.put("specificModel", specificModel);
			map.put("unit", unit);
			map.put("amount", amount);
			map.put("price", price);
			map.put("sum", sum);
			contentList.add(map);
		}
		return contentList;
	}
	
	protected void checkSubclass() {
	}
}
