package com.gx.obe.evaluation.composite;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.listener.ChangeTextListener;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.utils.MenuFunctionUtils;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.evaluation.composite.TenderProjectListComposite.ProjectEvaluateListener;
import com.gx.obe.listener.ProjectItemMouseEnterListener;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.vo.TenderProjectVo;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 项目界面面板
 * @author guoyr
 */
public class TenderProjectItemComposite extends Composite {

	private Label indexLabel;
	private Label tenderNoLabel;
	private ELink tenderNameLabel;
	private CLabel phaseTypeLabel;
	private Label countTitle_label;
	private Label countValue_label;
	private Label tenderTypeValueLabel;
	private Label tenderMethodValueLabel;
	private Label bidOpeningTimeValueLabel;
	private Label evalTimeValueLabel;
	public static final String  FORMAT = "yyyy-MM-dd HH:mm";
	private ProjectEvaluateListener listener;
	private int childCount = 0;
	private int supplierCount = 0;
	private ELink goto_btn;
	private TenderProjectEntity tenderProject;
	private Composite bidOpeningTimeComposite;
	private Composite evalTimeComposite;
	private Composite infoComposite;
	private CLabel label_7;
	private MenuFunctionUtils menuFunctions;
	private Composite evalResultComposite;
	private Label evalStatusLabel;
	private ImageButton evalUploadStatusLabel;

	/**
	 * @param parent
	 * @param style
	 */
	public TenderProjectItemComposite(Composite parent, MenuFunctionUtils menuFunctions, int style) {
		super(parent, style);
		this.menuFunctions = menuFunctions;
		setBackgroundMode(SWT.INHERIT_FORCE);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 2;
		gridLayout.marginWidth = 2;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite.setLayout(new GridLayout(4, false));
		GridData gd_composite = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_composite.widthHint = 700;
		composite.setLayoutData(gd_composite);
		
		Composite composite_2 = new Composite(composite, SWT.NONE);
		composite_2.setBackgroundMode(SWT.INHERIT_FORCE);
		composite_2.setBackground(SWTResourceManager.getColor(255, 153, 51));
		composite_2.setLayout(new GridLayout(1, false));
		
		indexLabel = new Label(composite_2, SWT.NONE);
		indexLabel.setAlignment(SWT.CENTER);
		indexLabel.setFont(SWTResourceManager.getFont("微软雅黑", 12, SWT.BOLD));
		indexLabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		
		tenderNoLabel = new Label(composite, SWT.NONE);
		tenderNoLabel.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
		tenderNoLabel.setForeground(ColorConstants.FONT_BLACK_COLOR);
		tenderNoLabel.setText("");
		
		tenderNameLabel = new ELink(composite, SWT.NONE);
		tenderNameLabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_BLACK));
		tenderNameLabel.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
		tenderNameLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Composite op_composite = new Composite(composite, SWT.NONE);
		op_composite.setLayout(new GridLayout(1, false));
		op_composite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		
		goto_btn = new ELink(op_composite, SWT.NONE);
		goto_btn.setActiveColor(SWTResourceManager.getColor(255, 153, 0));
		goto_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/evaluation.png"));
		goto_btn.setText(Messages.Enter);
		
		Label label = new Label(this, SWT.NONE);
		label.setBackground(SWTResourceManager.getColor(240, 240, 240));
		GridData gd_label = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_label.heightHint = 1;
		label.setLayoutData(gd_label);
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_1 = new GridLayout(4, false);
		gl_composite_1.horizontalSpacing = 0;
		gl_composite_1.marginWidth = 0;
		gl_composite_1.marginHeight = 0;
		composite_1.setLayout(gl_composite_1);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		menuFunctions.addFunction(composite_1, "PHASE_TYPE");
		
		phaseTypeLabel = new CLabel(composite_1, SWT.NONE);
		phaseTypeLabel.setAlignment(SWT.CENTER);
		GridData gd_phaseTypeLabel = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_phaseTypeLabel.widthHint = 32;
		phaseTypeLabel.setLayoutData(gd_phaseTypeLabel);
		phaseTypeLabel.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/phase_type.png"));
		phaseTypeLabel.setText("");
		
		CLabel label_4 = new CLabel(composite_1, SWT.NONE);
		label_4.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/gv.png"));
		label_4.setText("");
		
		infoComposite = new Composite(composite_1, SWT.NONE);
		GridLayout gl_infoComposite = new GridLayout(11, false);
		gl_infoComposite.marginWidth = 0;
		gl_infoComposite.marginHeight = 0;
		infoComposite.setLayout(gl_infoComposite);
		infoComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Composite subPackCountComposite = new Composite(infoComposite, SWT.NONE);
		subPackCountComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		GridLayout gl_subPackCountComposite = new GridLayout(2, false);
		gl_subPackCountComposite.horizontalSpacing = 0;
		gl_subPackCountComposite.marginWidth = 0;
		gl_subPackCountComposite.marginHeight = 0;
		subPackCountComposite.setLayout(gl_subPackCountComposite);
		menuFunctions.addFunction(subPackCountComposite, "COUNT");
		
		countTitle_label = new Label(subPackCountComposite, SWT.NONE);
		countTitle_label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		countTitle_label.setForeground(ColorConstants.FONT_BLACK_COLOR);
		countTitle_label.setText("分 包 数：");
		
		countValue_label = new Label(subPackCountComposite, SWT.NONE);
		countValue_label.setForeground(SWTResourceManager.getColor(72, 152, 21));
//		countValue_label.setText("2个");
		
		CLabel lab_5 = new CLabel(infoComposite, SWT.NONE);
		lab_5.setAlignment(SWT.CENTER);
		GridData gd_lab_5 = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_lab_5.widthHint = 20;
		lab_5.setLayoutData(gd_lab_5);
		lab_5.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/v_gray.png"));
		lab_5.setText("");
		menuFunctions.addFunction(lab_5, "TENDER_TYPE");
		
		Composite tenderTypeComposite = new Composite(infoComposite, SWT.NONE);
		subPackCountComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		GridLayout gl_tenderTypeComposite = new GridLayout(2, false);
		gl_tenderTypeComposite.horizontalSpacing = 0;
		gl_tenderTypeComposite.marginWidth = 0;
		gl_tenderTypeComposite.marginHeight = 0;
		tenderTypeComposite.setLayout(gl_tenderTypeComposite);
		menuFunctions.addFunction(tenderTypeComposite, "TENDER_TYPE");
		
		Label tenderTypeLabel = new Label(tenderTypeComposite, SWT.NONE);
		tenderTypeLabel.setForeground(ColorConstants.FONT_BLACK_COLOR);
		tenderTypeLabel.setText(Messages.TenderType + "：");
		
		tenderTypeValueLabel = new Label(tenderTypeComposite, SWT.NONE);
		tenderTypeValueLabel.setForeground(SWTResourceManager.getColor(72, 152, 21));
//		tenderTypeValueLabel.setText("政府采购");
		
		CLabel label_5 = new CLabel(infoComposite, SWT.NONE);
		label_5.setAlignment(SWT.CENTER);
		GridData gd_label_5 = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_label_5.widthHint = 20;
		label_5.setLayoutData(gd_label_5);
		label_5.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/v_gray.png"));
		label_5.setText("");
		menuFunctions.addFunction(label_5, "TENDER_METHOD");
		
		Composite tenderMethodComposite = new Composite(infoComposite, SWT.NONE);
		subPackCountComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		GridLayout gl_tenderMethodComposite = new GridLayout(2, false);
		gl_tenderMethodComposite.horizontalSpacing = 0;
		gl_tenderMethodComposite.marginWidth = 0;
		gl_tenderMethodComposite.marginHeight = 0;
		tenderMethodComposite.setLayout(gl_tenderMethodComposite);
		menuFunctions.addFunction(tenderMethodComposite, "TENDER_METHOD");
		
		Label tenderMethodLabel = new Label(tenderMethodComposite, SWT.NONE);
		tenderMethodLabel.setForeground(ColorConstants.FONT_BLACK_COLOR);
		tenderMethodLabel.setText(Messages.TenderMethod+"：");
		
		tenderMethodValueLabel = new Label(tenderMethodComposite, SWT.NONE);
		tenderMethodValueLabel.setForeground(SWTResourceManager.getColor(72, 152, 21));
//		tenderMethodValueLabel.setText("公开招标   ");
		
		CLabel label_6 = new CLabel(infoComposite, SWT.NONE);
		label_6.setAlignment(SWT.CENTER);
		GridData gd_label_6 = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_label_6.widthHint = 20;
		label_6.setLayoutData(gd_label_6);
		label_6.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/v_gray.png"));
		label_6.setText("");
		menuFunctions.addFunction(label_6, "OPENING_TIME");
		
		bidOpeningTimeComposite = new Composite(infoComposite, SWT.NONE);
		subPackCountComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		GridLayout gl_bidOpeningTimeComposite = new GridLayout(2, false);
		gl_bidOpeningTimeComposite.horizontalSpacing = 0;
		gl_bidOpeningTimeComposite.marginWidth = 0;
		gl_bidOpeningTimeComposite.marginHeight = 0;
		bidOpeningTimeComposite.setLayout(gl_bidOpeningTimeComposite);
		menuFunctions.addFunction(bidOpeningTimeComposite, "OPENING_TIME");
		
		Label bidOpeningTimeLabel = new Label(bidOpeningTimeComposite, SWT.NONE);
		bidOpeningTimeLabel.setForeground(ColorConstants.FONT_BLACK_COLOR);
		bidOpeningTimeLabel.setText(Messages.BidOpeningTime+"：");
		
		bidOpeningTimeValueLabel = new Label(bidOpeningTimeComposite, SWT.NONE);
		bidOpeningTimeValueLabel.setForeground(SWTResourceManager.getColor(72, 152, 21));
//		bidOpeningTimeValueLabel.setText("2016-01-17 09:00");
		
		label_7 = new CLabel(infoComposite, SWT.NONE);
		label_7.setAlignment(SWT.CENTER);
		GridData gd_label_7 = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_label_7.widthHint = 20;
		label_7.setLayoutData(gd_label_7);
		label_7.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/v_gray.png"));
		label_7.setText("");
		menuFunctions.addFunction(label_7, "EVAL_TIME");
		
		evalTimeComposite = new Composite(infoComposite, SWT.NONE);
		subPackCountComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		GridLayout gl_evalTimeComposite = new GridLayout(2, false);
		gl_evalTimeComposite.horizontalSpacing = 0;
		gl_evalTimeComposite.marginWidth = 0;
		gl_evalTimeComposite.marginHeight = 0;
		evalTimeComposite.setLayout(gl_evalTimeComposite);
		menuFunctions.addFunction(label_7, "EVAL_TIME");
		
		Label evalTimeLabel = new Label(evalTimeComposite, SWT.NONE);
		evalTimeLabel.setText(Messages.EvaluationTime+"：");
		
		evalTimeValueLabel = new Label(evalTimeComposite, SWT.NONE);
		evalTimeValueLabel.setForeground(SWTResourceManager.getColor(72, 152, 21));
		
		CLabel evalStatusSep = new CLabel(infoComposite, SWT.NONE);
		evalStatusSep.setAlignment(SWT.CENTER);
		GridData gd_evalStatusSep = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_evalStatusSep.widthHint = 20;
		evalStatusSep.setLayoutData(gd_evalStatusSep);
		evalStatusSep.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/v_gray.png"));
		evalStatusSep.setText("");
		menuFunctions.addFunction(evalStatusSep, "EVALUATION_STATUS");
		
		evalResultComposite = new Composite(infoComposite, SWT.NONE);
		subPackCountComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		GridLayout gl_evalResultComposite = new GridLayout(3, false);
		gl_evalResultComposite.horizontalSpacing = 0;
		gl_evalResultComposite.marginWidth = 0;
		gl_evalResultComposite.marginHeight = 0;
		evalResultComposite.setLayout(gl_evalResultComposite);
		menuFunctions.addFunction(evalResultComposite, "EVALUATION_STATUS");
		
		Label evalResultLabel = new Label(evalResultComposite, SWT.NONE);
		evalResultLabel.setText(Messages.EvaluationStatus + "：");
		evalResultLabel.setForeground(ColorConstants.FONT_BLACK_COLOR);
		
		evalStatusLabel = new Label(evalResultComposite, SWT.NONE);
		evalStatusLabel.setForeground(SWTResourceManager.getColor(100, 100, 100));
//		evalStatusLabel.setText(Messages.Progress_noStart);
		
		evalUploadStatusLabel = new ImageButton(evalResultComposite, SWT.NONE);
		evalUploadStatusLabel.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/update.png"));
		
		init();
	}

	/**
	 * @Description: 初始化
	 * @author guoyr
	 */
	public void init(){
		ProjectItemMouseEnterListener myMouseTrackListener = new ProjectItemMouseEnterListener(this);
		setBackgroundColor(this, myMouseTrackListener);
		EnterListener enterListener = new EnterListener();
		tenderNoLabel.addMouseListener(new ChangeTextListener());
		tenderNameLabel.addMouseListener(enterListener);
		goto_btn.addMouseListener(enterListener);
		menuFunctions.refreshFunctionAuthority();
	}
	/**
	 * @Description: 递归设置当前面板的鼠标进入时整个面板的背景颜色
	 * @author guoyr
	 * @param targetComposite
	 * @param myMouseTrackListener 
	 */
	private void setBackgroundColor(Composite targetComposite, ProjectItemMouseEnterListener myMouseTrackListener){
		targetComposite.addMouseTrackListener(myMouseTrackListener);
		for(Control contrl : targetComposite.getChildren()){
			if(contrl instanceof Composite){
				setBackgroundColor((Composite)contrl, myMouseTrackListener);
			}else{
				contrl.addMouseTrackListener(myMouseTrackListener);
			}
		}
	}
	class EnterListener extends MouseAdapter{
		@Override
		public void mouseUp(MouseEvent e) {
			enterProjectAction();
			super.mouseUp(e);
		}
	}
	/**
	 * @Description: 进入项目事件
	 * @author guoyr
	 */
	public void enterProjectAction(){
		listener.enterProject(tenderProject);
	}
	
	public void addProjectOpListener(ProjectEvaluateListener listener){
		this.listener = listener;
	}

	/**
	 * @Description: 刷新数据
	 * @author guoyr
	 * @param index
	 * @param resultMap
	 */
	public void refresh(int index, TenderProjectVo tenderProjectVo){
		tenderProject = new TenderProjectEntity();
		tenderProject.setId(tenderProjectVo.getTenderId());
		tenderProject.setTenderNo(tenderProjectVo.getTenderNo());
		tenderProject.setTenderName(tenderProjectVo.getTenderName());
		tenderProject.setTenderType(tenderProjectVo.getTenderType());
		tenderProject.setPhaseType(tenderProjectVo.getPhaseType());
		tenderProject.setIsPack(tenderProjectVo.getIsPack());
		tenderProject.setTenderMethod(tenderProjectVo.getTenderMethod());
		childCount  = tenderProjectVo.getChildCount();
		tenderProject.setChildrenCount(childCount);
		if(index < 10){
			indexLabel.setText("0"+index);
		}else {
			indexLabel.setText(""+index);
		}
		goto_btn.setText(Messages.Enter);
		goto_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/evaluation.png"));
		if(CommonEnum.YES.equals(tenderProject.getIsPack())){
//			goto_btn.setText(Messages.Enter);
			countTitle_label.setText(com.gx.obe.components.core.I18N.getString(I18NEnum.SUBPACKAGE, tenderProject.getTenderType()) + "：");
			countValue_label.setText(childCount+Messages.NumCouns);
			WidgetUtils.setControlVisible(evalUploadStatusLabel, false);
			if(tenderProjectVo.getEvaluationComplateCount() > 0 && tenderProjectVo.getChildCount() > 0){
				evalStatusLabel.setForeground(SWTResourceManager.getColor(72, 152, 21));
				if(tenderProjectVo.getOpeningComplateCount() >= tenderProjectVo.getChildCount()) {
					evalStatusLabel.setText(Messages.Progress_complate);
				}else {
					evalStatusLabel.setText(Messages.Progress_complate +"  "+(int)(tenderProjectVo.getOpeningComplateCount() * 100d /tenderProjectVo.getChildCount())+"%");
				}
			}else {
				evalStatusLabel.setText(Messages.Progress_complate +"  0%");
			}
		}else {
			countTitle_label.setText(com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, tenderProject.getTenderType()) +"：");
			supplierCount  = tenderProjectVo.getSupplierCount();
			countValue_label.setText(supplierCount+Messages.SupplierCouns);
			evalUploadStatusLabel.setEnabled(false);
			if(CommonEnum.COMPLETE.equals(tenderProjectVo.getComplateStatus())){
				if(CommonEnum.COMPLETE.equals(tenderProjectVo.getUploadEvaluationResultStatus())){
					evalUploadStatusLabel.setEnabled(true);
				}
				evalStatusLabel.setText(Messages.Progress_complate);
				evalStatusLabel.setForeground(SWTResourceManager.getColor(72, 152, 21));
			}else if(StringUtils.isEmpty(tenderProjectVo.getComplateStatus()) || CommonEnum.NOT_START.equals(tenderProjectVo.getComplateStatus())){
				evalStatusLabel.setText(Messages.Progress_noStart);
			}else {
				evalStatusLabel.setText(Messages.Progress_noComplate);
			}
		}
		
		if(TenderProjectEnum.PHASE_TYPE_PRE.equals(tenderProject.getPhaseType())){
			phaseTypeLabel.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/phase_type.png"));
		}else {
			phaseTypeLabel.setImage(null);
		}
		tenderNoLabel.setText("["+tenderProject.getTenderNo() + "]");
		tenderNameLabel.setText(tenderProject.getTenderName());
		tenderTypeValueLabel.setText(TenderProjectEnum.getProjectTypeCN(tenderProject.getTenderType()));
		tenderMethodValueLabel.setText(TenderProjectEnum.getTenderMethodCN(tenderProject.getTenderMethod()));
		bidOpeningTimeValueLabel.setText(ObjectUtils.getObjString(DateUtils.getFormat(tenderProjectVo.getBidOpeningStartTime(), FORMAT)));
		evalTimeValueLabel.setText(ObjectUtils.getObjString(DateUtils.getFormat(tenderProjectVo.getBidEvaluationStartTime(), FORMAT)));
		infoComposite.layout();
	
	}
	
	@Override
	protected void checkSubclass() {
	}
}
