package com.gx.obe.evaluation.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ELabel;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.ClipboardUtils;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 符合性评审节点信息
 * @author guoyr
 */
public class ComplianceFactorInfoDialog extends EDialog {
	
	private Text factorMemoText;
	private EvaluationFactor evaluationFactor;
	private ELabel factorNameLabel;
	private ELabel objectiveLabel;
	
	{
		LogUtils.logClass(ComplianceFactorInfoDialog.class);
	}
	
	/**
	 * @param parentShell
	 * @param evaluationFactor
	 */
	public ComplianceFactorInfoDialog(Shell parentShell, EvaluationFactor evaluationFactor) {
		super(parentShell);
		setTitle("指标详细信息");
		this.evaluationFactor = evaluationFactor;
	}

	/** 
	 * @Description: 
	 * @param parent
	 * @return
	 */
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		Composite container3 = new Composite(container, SWT.NONE);
		container3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_container3 = new GridLayout(3, false);
		container3.setLayout(gl_container3);
		
		Label label = new Label(container3, SWT.NONE);
		label.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		label.setText("指标名称：");
		
		factorNameLabel = new ELabel(container3, SWT.WRAP);
		factorNameLabel.setFont(SWTResourceManager.getFont("宋体", 10, SWT.BOLD));
		factorNameLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		factorNameLabel.setText("投标资格是否满足招标文件要求");
		
		ELink label_3 = new ELink(container3, SWT.NONE);
		label_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label_3.setText("复制");
		label_3.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/copy.png"));
		label_3.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				ClipboardUtils.copy(factorNameLabel.getText());
			}
		});
		
		Label label_2 = new Label(container3, SWT.NONE);
		label_2.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, true, 1, 1));
		label_2.setText("指标说明：");
		
		factorMemoText = new Text(container3, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		factorMemoText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		
		objectiveLabel = new ELabel(container3, SWT.WRAP);
		objectiveLabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GRAY));
		objectiveLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));
		objectiveLabel.setText("当前指标为“客观项指标”，所有专家对此项的评标结果须一致！");
		
		ESepator label_1 = new ESepator(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label_1.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite = new Composite(container, SWT.NONE);
		composite.setBackgroundMode(SWT.INHERIT_FORCE);
		composite.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));
		
		EButton button = new EButton(composite, SWT.NONE);
		button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		button.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				getShell().close();
			}
		});
		GridData gd_button = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		button.setLayoutData(gd_button);
		button.setText(Messages.Button_close);

		init();
		return container;
	}

	/**
	 * @Description: 界面数据初始化
	 * @author guoyr
	 */
	private void init(){
		if(null == evaluationFactor){
			evaluationFactor = new EvaluationFactor();
		}
		factorNameLabel.setText(ObjectUtils.getObjString(evaluationFactor.getFactorName()));
		factorMemoText.setText(ObjectUtils.getObjString(evaluationFactor.getMemo()));
		if(!ObjectUtils.equals(evaluationFactor.getObjective(), EvaluationFactorEnum.OBJECTIVE_YES)){
			objectiveLabel.dispose();
		}
	}
	protected Point getInitialSize() {
		return new Point(398, 227);
	}
}
