package com.gx.obe.evaluation.dialog;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.utils.SwtUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ELabel;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.business.entity.ScoreItem;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.ClipboardUtils;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.SerializeUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 详细评审评标步骤信息
 * @author guoyr
 */
public class ScoreFactorInfoDialog extends EDialog {
	
	private Text factorMemoText;
	private EvaluationFactor evaluationFactor;
	private ELabel factorNameLabel;
	private ELabel objectiveLabel;
	private Label scoreLabel;
	private Label weightLabel;
	private Label rangeLabel;
	private Grid objectiveItemGrid;
	private CLabel scoreTypeCLabel;
	private Composite composite_3;
	
	{
		LogUtils.logClass(ScoreFactorInfoDialog.class);
	}
	
	/**
	 * @param parentShell
	 * @param evaluationFactor
	 */
	public ScoreFactorInfoDialog(Shell parentShell, EvaluationFactor evaluationFactor) {
		super(parentShell);
		setTitle("指标详细信息");
		this.evaluationFactor = evaluationFactor;
	}

	/** 
	 * @Description: 
	 * @param parent
	 * @return
	 */
	protected Control createDialogArea(Composite parent) {
		Composite top = (Composite) super.createDialogArea(parent);
		GridLayout gl_top = new GridLayout(1, false);
		gl_top.verticalSpacing = 0;
		gl_top.horizontalSpacing = 0;
		gl_top.marginHeight = 0;
		gl_top.marginWidth = 0;
		top.setLayout(gl_top);
		
		Composite container = new Composite(top, SWT.NONE);
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_container = new GridLayout(3, false);
		gl_container.marginWidth = 10;
		container.setLayout(gl_container);
		
		Label label = new Label(container, SWT.NONE);
		label.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		label.setText("指标名称：");
		
		factorNameLabel = new ELabel(container, SWT.WRAP);
		factorNameLabel.setFont(SWTResourceManager.getFont("宋体", 10, SWT.BOLD));
		factorNameLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		factorNameLabel.setText("投标资格是否满足招标文件要求");
		
		ELink label_3 = new ELink(container, SWT.NONE);
		label_3.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				ClipboardUtils.copy(factorNameLabel.getText());
			}
		});
		label_3.setText("复制");
		label_3.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/copy.png"));
		label_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		
		Label label_1 = new Label(container, SWT.NONE);
		label_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		label_1.setText("分值：");
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1));
		GridLayout gl_composite_1 = new GridLayout(3, false);
		gl_composite_1.marginHeight = 0;
		gl_composite_1.marginWidth = 0;
		composite_1.setLayout(gl_composite_1);
		
		scoreLabel = new Label(composite_1, SWT.NONE);
		GridData gd_scoreLabel = new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1);
		gd_scoreLabel.widthHint = 60;
		scoreLabel.setLayoutData(gd_scoreLabel);
		scoreLabel.setText("15");
		
		weightLabel = new Label(composite_1, SWT.NONE);
		GridData gd_weightLabel = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		gd_weightLabel.widthHint = 87;
		weightLabel.setLayoutData(gd_weightLabel);
		weightLabel.setText("20%");
		
		Label label_4 = new Label(container, SWT.NONE);
		label_4.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		label_4.setText("评分范围：");
		
		rangeLabel = new Label(container, SWT.NONE);
		rangeLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		rangeLabel.setText("0 - 15");
		new Label(container, SWT.NONE);
		
		Label label_5 = new Label(container, SWT.NONE);
		label_5.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
		label_5.setText("打分方式：");
		
		Composite composite_2 = new Composite(container, SWT.NONE);
		GridLayout gl_composite_2 = new GridLayout(1, false);
		gl_composite_2.marginHeight = 0;
		gl_composite_2.marginWidth = 0;
		composite_2.setLayout(gl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
		
		scoreTypeCLabel = new CLabel(composite_2, SWT.NONE);
		scoreTypeCLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		scoreTypeCLabel.setText("客观范围打分");
		
		composite_3 = new Composite(composite_2, SWT.NONE);
		composite_3.setLayout(new FillLayout(SWT.HORIZONTAL));
		GridData gd_composite_3 = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_composite_3.heightHint = 80;
		composite_3.setLayoutData(gd_composite_3);
		
		objectiveItemGrid = new Grid(composite_3, SWT.BORDER | SWT.V_SCROLL);
		objectiveItemGrid.setHeaderVisible(true);
		
		GridColumn gridColumn_1 = new GridColumn(objectiveItemGrid, SWT.CENTER);
		gridColumn_1.setText(Messages.Num);
		gridColumn_1.setWidth(40);
		gridColumn_1.setMaxmumWidth(40);
		
		GridColumn gridColumn_2 = new GridColumn(objectiveItemGrid, SWT.NONE);
		gridColumn_2.setText("打分说明");
		gridColumn_2.setWidth(210);
		gridColumn_2.setWordWrap(true);
		
		GridColumn gridColumn_3 = new GridColumn(objectiveItemGrid, SWT.RIGHT);
		gridColumn_3.setText("得分");
		gridColumn_3.setWidth(80);
		
		Label label_2 = new Label(container, SWT.NONE);
		label_2.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, true, 1, 1));
		label_2.setText("指标说明：");
		
		factorMemoText = new Text(container, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		GridData gd_factorMemoText = new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1);
		gd_factorMemoText.heightHint = 50;
		factorMemoText.setLayoutData(gd_factorMemoText);
		
		objectiveLabel = new ELabel(container, SWT.WRAP);
		objectiveLabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GRAY));
		objectiveLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));
		objectiveLabel.setText("当前指标为“客观项指标”，所有专家对此项的评分结果须一致！");
		
		ESepator label_6 = new ESepator(top, SWT.SEPARATOR | SWT.HORIZONTAL);
		label_6.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label_6.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite = new Composite(top, SWT.NONE);
		composite.setBackgroundMode(SWT.INHERIT_FORCE);
		composite.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));
		
		EButton cancel_btn = new EButton(composite, SWT.NONE);
		cancel_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		cancel_btn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				getShell().close();
			}
		});
		GridData gd_button = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		cancel_btn.setLayoutData(gd_button);
		cancel_btn.setText(Messages.Button_close);

		init();
		return top;
	}

	/**
	 * @Description: 界面数据初始化
	 * @author guoyr
	 */
	private void init(){
		if(null == evaluationFactor){
			evaluationFactor = new EvaluationFactor();
		}
		factorNameLabel.setText(ObjectUtils.getObjString(evaluationFactor.getFactorName()));
		factorMemoText.setText(ObjectUtils.getObjString(evaluationFactor.getMemo()));
		scoreLabel.setText(null != evaluationFactor.getFactorScore()? BigDecimalUtils.round(evaluationFactor.getFactorScore(), 2).toString():"");
		if(null != evaluationFactor.getFactorWeight()){
			weightLabel.setText(BigDecimalUtils.round(evaluationFactor.getFactorWeight(), 2).toString() +"%");
		}else {
			weightLabel.dispose();
		}
		StringBuilder rangel = new StringBuilder();
		rangel.append(null != evaluationFactor.getFactorMinScore()? BigDecimalUtils.round(evaluationFactor.getFactorMinScore(), 2).toString():"0");
		rangel.append("  -  ");
		rangel.append(null != evaluationFactor.getFactorMaxScore()? BigDecimalUtils.round(evaluationFactor.getFactorMaxScore(), 2).toString():scoreLabel.getText());
		rangeLabel.setText(rangel.toString());
		if(!ObjectUtils.equals(evaluationFactor.getObjective(), EvaluationFactorEnum.OBJECTIVE_YES)){
			objectiveLabel.dispose();
		}
		scoreTypeCLabel.setText(EvaluationFactorEnum.getScoreTypeCN(evaluationFactor.getScoreType()));
		if(StringUtils.equals(EvaluationFactorEnum.SCORE_TYPE_OBJECTIVE, evaluationFactor.getScoreType())){
			// 客观范围打分
			objectiveItemGrid.setAutoWidth(true);
			objectiveItemGrid.setAutoHeight(true);
			objectiveItemGrid.resizeColumnWidth();
			refreshObjectiveItem(evaluationFactor);
		}else{
			composite_3.dispose();
		}
		getShell().setMinimumSize(400, 280);
		getShell().pack();
		SwtUtils.setCenter(getShell());
	}
	
	/**
	 * @Description: 刷新客观范围打分数据
	 * @author guoyr
	 * @param selectFactor
	 */
	@SuppressWarnings("unchecked")
	private void refreshObjectiveItem(EvaluationFactor selectFactor){
		objectiveItemGrid.removeAll();
		List<ScoreItem> objectiveItemList = new ArrayList<ScoreItem>();
		if(null != selectFactor){
			objectiveItemList = (List<ScoreItem>) SerializeUtils.deserializeObject(selectFactor.getObjectiveItem());
		}
		if(null != objectiveItemList && objectiveItemList.size() > 0){
			int index = 1;
			for(ScoreItem item : objectiveItemList){
				GridItem gridItem = new GridItem(objectiveItemGrid, SWT.NONE);
				gridItem.setText(0, ObjectUtils.getObjString(index++));
				initObjectiveItemGridItem(gridItem, item);
			}
		}
	}
	
	/**
	 * @Description: 初始化客观范围打分GridItem
	 * @author guoyr
	 * @param gridItem
	 * @param item
	 */
	private void initObjectiveItemGridItem(GridItem gridItem, ScoreItem item){
		gridItem.setText(1, ObjectUtils.getObjString(item.getName()));
		gridItem.setText(2, ObjectUtils.getObjString(item.getScore()));
		gridItem.setData("scoreItem", item);
	}
	@Override
	protected Point getInitialSize() {
		return new Point(444, 348);
		
	}
}
