package com.gx.obe.evaluation.dialog;

import java.util.List;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.GridUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.button.Tips;
import com.gx.obe.components.core.Constants;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.auth.AuthUser;
import com.gx.obe.web.service.expert.ExpertService;
import com.swtdesigner.ResourceManager;

/**
 * @Description:
 * @author guoyr
 */
public class SelectUserDialog extends EDialog {
	
	private final ExpertService expertService = new ExpertService();
	
	private final String USER = "user";
	
	private String tempUserId;
	private List<AuthUser> userList;
	
	private Grid userGrid;

	private AuthUser selectUser;

	{
		LogUtils.logClass(SelectUserDialog.class);
	}
	
	/**
	 * @author chenxw
	 * @param parentShell
	 * @param tempUser
	 * @param userList
	 */
	public SelectUserDialog(Shell parentShell, String tempUserId, List<AuthUser> userList) {
		super(parentShell);
		this.setTitle("选择用户");
		this.setShellStyle(SWT.MAX | SWT.CLOSE);
		this.tempUserId = tempUserId;
		this.userList = userList;
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.marginWidth = 0;
		gl_container.marginHeight = 0;
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		container.setLayout(gl_container);
		
		Composite toolComposite = new Composite(container, SWT.None);
		toolComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_toolComposite = new GridLayout(1, false);
		gl_toolComposite.verticalSpacing = 0;
		gl_toolComposite.horizontalSpacing = 0;
		toolComposite.setLayout(gl_toolComposite);
		
		Tips tips = new Tips(toolComposite, SWT.NONE);
		tips.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		tips.setText("检测到您的用户信息已经存在，请您确认选择个人用户信息！");
		
		userGrid = new Grid(container, SWT.V_SCROLL);
		userGrid.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GridItem item = (GridItem) e.item;
				GridUtils.checkAll(userGrid, 0, false);
				item.setChecked(0, true);
			}
		});
		userGrid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		userGrid.setHeaderVisible(true);
		userGrid.setAutoWidth(true);
		userGrid.setItemHeight(Constants.ROW_HIGHT);
		
		GridColumn columnNo = new GridColumn(userGrid, SWT.CENTER | SWT.CHECK);
		columnNo.setWidth(50);
		columnNo.setMaxmumWidth(50);
		columnNo.setText("序号");
		
		GridColumn userNamecolumn = new GridColumn(userGrid, SWT.CENTER);
		userNamecolumn.setWidth(150);
		userNamecolumn.setMaxmumWidth(150);
		userNamecolumn.setText("姓名");
		
		GridColumn accountcolumn = new GridColumn(userGrid, SWT.CENTER);
		accountcolumn.setWidth(150);
		accountcolumn.setMaxmumWidth(150);
		accountcolumn.setText("账号");
		
		GridColumn idCardcolumn = new GridColumn(userGrid, SWT.CENTER);
		idCardcolumn.setWidth(200);
		idCardcolumn.setMaxmumWidth(200);
		idCardcolumn.setText("身份证号");
		
		new GridColumn(userGrid, SWT.CENTER);
		
		ESepator label_2 = new ESepator(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		label_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label_2.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite = new Composite(container, SWT.NONE);
		composite.setLayout(new GridLayout(2, false));
		composite.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		
		EButton button = new EButton(composite, SWT.NONE);
		button.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				selectUserAction();
			}
		});
		GridData gd_button = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_button.widthHint = 200;
		button.setLayoutData(gd_button);
		button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/ok.png"));
		button.setText("确认选择");
		
		EButton button_1 = new EButton(composite, SWT.NONE);
		button_1.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				close();
			}
		});
		GridData gd_button_1 = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_button_1.widthHint = 200;
		button_1.setLayoutData(gd_button_1);
		button_1.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		button_1.setText("不，这些都不是我个人信息");
		
		init();
		
		return container;
	}
	
	/**
	 * 界面数据初始化
	 */
	private void init() {
		if (CollectionUtils.isNull(userList)) {
			return;
		}
		int index = 1;
		for (AuthUser user : userList) {
			GridItem item = new GridItem(userGrid, SWT.NULL);
			item.setCheckable(0, false);
			item.setText(0, (index++) + "");
			item.setText(1, user.getUserName());
			item.setText(2, user.getUserAccount());
			item.setText(3, ObjectUtils.getObjString(user.getUserIdcard()));
			item.setData(USER, user);
		}
		if (userList.size() == 1) {
			userGrid.setSelection(0);
		}
	}
	
	/**
	 * @Description: 选择当前用户
	 * @author chenxw
	 */
	private void selectUserAction() {
		List<GridItem> itemList = GridUtils.getCheckedItems(userGrid, 0);
		if (CollectionUtils.isNull(itemList)) {
			MessageDialog.openError(getShell(), "请选择符合您的用户信息！");
			return;
		}
		
		AuthUser selectUser = (AuthUser) itemList.get(0).getData(USER);
		
		if (expertService.isTenderExpertContain(selectUser.getId(), tempUserId)) {
			MessageDialog.openError(getShell(), "项目下已存在账号为[" + selectUser.getUserAccount() + "]的专家，请您直接使用该账号登录，或重新设置其它账号！");
			return;
		}
		
		UserVerificationDialog dialog = new UserVerificationDialog(getShell(), selectUser);
		dialog.open();
		if (!dialog.isVerification()) {
			return;
		}
		this.selectUser = selectUser;
		close();
	}
	
	public AuthUser getSelectUser() {
		return selectUser;
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(701, 464);
	}
	
}
