package com.gx.obe.evaluation.dialog;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.nebula.widgets.grid.listener.GridCheckedListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.CallBack;
import org.eclipse.swt.core.ImageResolver;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.GridUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.utils.ParsingXmlUtils;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.FreemarkerUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.evaluation.EvaluationFactorService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class ShowCompareEvaluationSupplierListDialog extends EDialog {

	private static Logger LOG = Logger.getLogger(ShowCompareEvaluationSupplierListDialog.class);
	
	private EButton contrastButton;
	private Grid evaluationGrid;
	private Grid supplierGrid;
	private EvaluationFactorService evaluationFactorService = new EvaluationFactorService();
	private TenderProjectEntity tenderProjectEntity;
	private EvaluationStep evaluationStep;
	private List<Supplier> supplierList;
	private Image menuOpenImage;
	private ImageResolver imageResolver = new ImageResolver();
	private Image menuImage;
	private GridColumn gridColumn;
	private Map<String, Object> dataMap;
	private EvaluationFactor curEvaluationFactor;
	private Supplier curSupplier;
	private CallBack callBack;

	/**
	 * Create the dialog.
	 * @param parentShell
	 * @param supplierList 
	 * @param curSupplier 
	 * @param curEvaluationFactor 
	 */
	public ShowCompareEvaluationSupplierListDialog(Shell parentShell, TenderProjectEntity tenderProjectEntity, EvaluationStep evaluationStep, List<Supplier> supplierList, EvaluationFactor curEvaluationFactor, Supplier curSupplier, CallBack callBack) {
		super(parentShell);
		setTitle("评审点对比");
		this.tenderProjectEntity = tenderProjectEntity;
		this.evaluationStep = evaluationStep;
		this.supplierList = supplierList;
		this.curEvaluationFactor = curEvaluationFactor;
		this.curSupplier = curSupplier;
		this.callBack = callBack;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, false));
		container.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				imageResolver.disposite();
			}
		});
		
		Composite composite = new Composite(container, SWT.NONE);
		
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginWidth = 0;
		composite.setLayout(gridLayout);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		Composite composite_2 = new Composite(composite, SWT.NONE);
		composite_2.setBackground(ColorConstants.BORDER_LINE_COLOR);
		GridLayout gl_composite_2 = new GridLayout(1, false);
		gl_composite_2.verticalSpacing = 1;
		gl_composite_2.horizontalSpacing = 1;
		gl_composite_2.marginHeight = 1;
		gl_composite_2.marginWidth = 1;
		composite_2.setLayout(gl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite composite_4 = new Composite(composite_2, SWT.NONE);
		composite_4.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_4.setLayout(new GridLayout(3, false));
		composite_4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label label = new Label(composite_4, SWT.NONE);
		label.setForeground(ColorConstants.FONT_BLACK_COLOR);
		label.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		label.setText("请选择需要对比的评审点！");
		
		ELink link = new ELink(composite_4, SWT.NONE);
		link.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		link.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		link.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/select_all.png"));
		link.setText(Messages.Button_selectAll);
		link.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				checkAllAction(evaluationGrid, true);
			}
		});
		
		ELink link_1 = new ELink(composite_4, SWT.NONE);
		link_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		link_1.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/disselect_all.png"));
		link_1.setText(Messages.Button_deselected);
		link_1.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				checkAllAction(evaluationGrid, false);
			}
		});
		
		Composite composite_5 = new Composite(composite_2, SWT.NONE);
		composite_5.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_5 = new GridLayout(1, false);
		gl_composite_5.verticalSpacing = 0;
		gl_composite_5.horizontalSpacing = 0;
		gl_composite_5.marginHeight = 0;
		gl_composite_5.marginWidth = 0;
		composite_5.setLayout(gl_composite_5);
		composite_5.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		evaluationGrid = new Grid(composite_5, SWT.H_SCROLL | SWT.V_SCROLL);
		evaluationGrid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		gridColumn = new GridColumn(evaluationGrid, SWT.CHECK);
		gridColumn.setText("New Column");
		gridColumn.setWidth(150);
		
		Composite composite_3 = new Composite(composite, SWT.NONE);
		composite_3.setBackground(ColorConstants.BORDER_LINE_COLOR);
		GridLayout gl_composite_3 = new GridLayout(1, false);
		gl_composite_3.verticalSpacing = 1;
		gl_composite_3.horizontalSpacing = 1;
		gl_composite_3.marginHeight = 1;
		gl_composite_3.marginWidth = 1;
		composite_3.setLayout(gl_composite_3);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite composite_6 = new Composite(composite_3, SWT.NONE);
		composite_6.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_6.setLayout(new GridLayout(3, false));
		composite_6.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label label_1 = new Label(composite_6, SWT.NONE);
		label_1.setForeground(ColorConstants.FONT_BLACK_COLOR);
		label_1.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		label_1.setText("请选择需要对比的供应商！");
		
		ELink link_2 = new ELink(composite_6, SWT.NONE);
		link_2.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		link_2.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		link_2.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/select_all.png"));
		link_2.setText(Messages.Button_selectAll);
		link_2.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				checkAllAction(supplierGrid, true);
			}
		});
		
		ELink link_3 = new ELink(composite_6, SWT.NONE);
		link_3.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		link_3.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/disselect_all.png"));
		link_3.setText(Messages.Button_deselected);
		link_3.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				checkAllAction(supplierGrid, false);
			}
		});
		
		Composite composite_7 = new Composite(composite_3, SWT.NONE);
		composite_7.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_7 = new GridLayout(1, false);
		gl_composite_7.verticalSpacing = 0;
		gl_composite_7.horizontalSpacing = 0;
		gl_composite_7.marginHeight = 0;
		gl_composite_7.marginWidth = 0;
		composite_7.setLayout(gl_composite_7);
		composite_7.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		supplierGrid = new Grid(composite_7, SWT.H_SCROLL | SWT.V_SCROLL);
		supplierGrid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		GridColumn gridColumn_1 = new GridColumn(supplierGrid, SWT.CHECK);
		gridColumn_1.setText("New Column");
		gridColumn_1.setWidth(150);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayout(new GridLayout(2, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		contrastButton = newEButton(composite_1, "对比");
		GridData gd_button = new GridData(SWT.RIGHT, SWT.FILL, true, true, 1, 1);
		gd_button.heightHint = 30;
		gd_button.widthHint = 150;
		contrastButton.setLayoutData(gd_button);
		contrastButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				contrastAction();
			}
		});
		
		EButton button_1 = newEButton(composite_1, Messages.Button_close);
		GridData gd_button_1 = new GridData(SWT.LEFT, SWT.FILL, true, true, 1, 1);
		gd_button_1.heightHint = 30;
		gd_button_1.widthHint = 150;
		button_1.setLayoutData(gd_button_1);
		button_1.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				close();
			}
		});
		
		init();
		return container;
	}
	
	private void init() {
		if (null == curEvaluationFactor) {
			curEvaluationFactor = new EvaluationFactor();
		}
		if (null == curSupplier) {
			curSupplier = new Supplier();
		}
		gridColumn.setShowTreeTogglePic(true);
		gridColumn.setWordWrap(true);
		gridColumn.setTreeTogglePic(ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_expend.png"), ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_close.png"));
		gridColumn.setTree(true);
		menuOpenImage = imageResolver.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_parent.png");
		menuImage = imageResolver.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_item.png");
		refreshFactorTree();
		refreshSupplierGrid();
		// 添加选中监听事件
		evaluationGrid.addListener(SWT.Selection, new GridCheckedListener(false));
		GridUtils.refreshGridCheckedStatus(evaluationGrid, 0);
		supplierGrid.addListener(SWT.Selection, new GridCheckedListener(false));
		GridUtils.refreshGridCheckedStatus(supplierGrid, 0);
	}
	
	/**
	 * @Description: 刷新指标树
	 * @author chenxw 
	 */
	private void refreshFactorTree() {
		evaluationGrid.removeAll();
		List<EvaluationFactor> evaluationFactorList = evaluationFactorService.getEvaluationStepFactorTreeList(tenderProjectEntity.getId(), evaluationStep);
		if (null != evaluationFactorList && evaluationFactorList.size() > 0) {
			GridItem topGridItem = new GridItem(evaluationGrid, SWT.NONE);
			topGridItem.setText(evaluationStep.getStepName());
			topGridItem.setImage(0, menuOpenImage);
			topGridItem.setExpanded(true);
			for (EvaluationFactor factor : evaluationFactorList) {
				GridItem gridItem = new GridItem(topGridItem, SWT.NONE);
				gridItem.setText(0, factor.getFactorName());
				gridItem.setData("evaluationFactors", factor);
				if (StringUtils.equals(factor.getId(), curEvaluationFactor.getId())) {
					gridItem.setChecked(0, true);
				}
				initChildGridItem(factor, gridItem);
			}
		} else {
			contrastButton.setEnabled(false);
		}
	}
	
	/**
	 * @Description: 指标信息封装树节点(双重循环实现拼接)
	 * @author guoyr
	 * @param list
	 * @return
	 */
	private List<EvaluationFactor> initChildFactor(List<EvaluationFactor> list){
		List<EvaluationFactor> factorList = null;
		if(null != list && list.size() > 0){
			factorList = new ArrayList<EvaluationFactor>();
			int topCount = 1;
			for(EvaluationFactor parentFactor : list){
				if(ObjectUtils.equals(parentFactor.getFactorType(), EvaluationFactorEnum.FACTOR_FLOAT)){
					continue;
				}
				int childCount = 0;
				parentFactor.getChildFactorList().clear();
				for(EvaluationFactor subFactor : list){
					// 如果存在包含指标，即存在浮动指标
					if(StringUtils.isNotEmpty(parentFactor.getContainId()) && subFactor.getId().equals(parentFactor.getContainId())){
						parentFactor.setContainFactor(subFactor);
						subFactor.setContainFactor(parentFactor);
					}
					if(StringUtils.isNotEmpty(subFactor.getParentId()) && parentFactor.getId().equals(subFactor.getParentId())){
						parentFactor.addChild(subFactor);
						subFactor.setSortNo(childCount + 1);
						subFactor.setParentFactor(parentFactor);
						childCount ++;
					}
				}
				// 如果是顶级指标
				if(StringUtils.isEmpty(parentFactor.getParentId())){
					parentFactor.setSortNo(topCount++);
					parentFactor.setChildCount(childCount);
					factorList.add(parentFactor);
				}
			}
		}
		return factorList;
	}
	
	/**
	 * @Description: 递归实现指标树
	 * @author guoyr
	 * @param factor
	 * @param parentGridItem
	 * @param selectId
	 */
	private void initChildGridItem(EvaluationFactor factor, GridItem parentGridItem) {
		if (null != factor.getChildFactorList() && factor.getChildFactorList().size() > 0) {
			for (EvaluationFactor childFactor : factor.getChildFactorList()) {
				GridItem gridItem = new GridItem(parentGridItem, SWT.NONE);
				gridItem.setText(0, childFactor.getFactorName());
				gridItem.setData("evaluationFactors", childFactor);
				if (StringUtils.equals(childFactor.getId(), curEvaluationFactor.getId())) {
					gridItem.setChecked(0, true);
				}
				initChildGridItem(childFactor, gridItem);
			}
			parentGridItem.setExpanded(true);
			parentGridItem.setImage(0, menuOpenImage);
		} else {
			parentGridItem.setImage(0, menuImage);
		}
		parentGridItem.setToolTipText(0, parentGridItem.getText());
	}
	
	/**
	 * @Description: 刷新供应商列表
	 * @author chenxw 
	 */
	private void refreshSupplierGrid() {
		supplierGrid.removeAll();
		if (null != supplierList && supplierList.size() > 0) {
			for (Supplier supplier : supplierList) {
				GridItem gridItem = new GridItem(supplierGrid, SWT.NONE);
				gridItem.setText(0, supplier.getSupplierName());
				gridItem.setData("supplier", supplier);
				if (StringUtils.equals(supplier.getId(), curSupplier.getId())) {
					gridItem.setChecked(0, true);
				}
			}
		}
	}
	
	/**
	 * @Description: 全选事件
	 * @author chenxw
	 * @param grid
	 * @param isCheck 
	 */
	protected void checkAllAction(Grid grid, boolean isCheck) {
		for (GridItem gridItem : grid.getItems()) {
			if (gridItem.getCheckable(0)) {
				gridItem.setChecked(0, isCheck);
			}
		}
	}

	
	/**
	 * @Description: 对比
	 * @author chenxw 
	 */
	protected void contrastAction() {
		if (!contrastButton.isEnabled()) {
			return;
		}
		
		List<EvaluationFactor> checkedEvaluationFactorList = GridUtils.getCheckedValues(evaluationGrid, 0, "evaluationFactors");
		if(null == checkedEvaluationFactorList || checkedEvaluationFactorList.size() == 0){
			MessageDialog.openWarning(getShell(), "请选择需要对比的评审指标！");
			return;
		}
		
		List<Supplier> checkedSupplierList = GridUtils.getRealCheckedValues(supplierGrid, 0, "supplier");
		if(null == checkedSupplierList || checkedSupplierList.size() == 0){
			MessageDialog.openWarning(getShell(), "请选择需要对比的供应商！");
			return;
		}
		
		Map<String, EvaluationFactor> supplierEvaluationFactorMap = null;
		try {
			supplierEvaluationFactorMap = getSupplierEvaluationFactorList(checkedSupplierList);
		} catch (Exception e) {
			MessageDialog.openWarning(getShell(), "解析供应商评审指标失败！");
			return;
		}
		if(null == supplierEvaluationFactorMap || supplierEvaluationFactorMap.size() == 0){
			MessageDialog.openWarning(getShell(), "没有解析到供应商评审指标！");
			return;
		}
		
		initDataMap(checkedEvaluationFactorList, checkedSupplierList, supplierEvaluationFactorMap);
		
		File stencilFile = new File(FileConstants.PRODUCT_ROOT + File.separator + FileConstants.RESOURCE_FOLDER + File.separator + "ShowCompareEvaluation.html");
		if (!stencilFile.exists()) {
			MessageDialog.openWarning(getShell(), "没有找到模版文件！");
			return;
		}
		
		String html;
		try {
			html = FreemarkerUtils.rendFreemarker(stencilFile.getParent(), stencilFile.getName(), dataMap);
		} catch (Exception e) {
			LOG.error("", e);
			MessageDialog.openWarning(getShell(), "模版加载出错！");
			return;
		}
		
		if (html.isEmpty()) {
			MessageDialog.openWarning(getShell(), "模版加载出错！");
			return;
		}
		
		callBack.callBackString(html);
		
		close();
	}

	private void initDataMap(List<EvaluationFactor> evaluationFactorList, List<Supplier> supplierList, Map<String, EvaluationFactor> supplierEvaluationFactorMap) {
		if (null == dataMap) {
			dataMap = new HashMap<String, Object>();
		} else {
			dataMap.clear();
		}
		
		evaluationFactorList = initChildFactor(evaluationFactorList);
		boolean isHasParent = false;
		for (EvaluationFactor evaluationFactor : evaluationFactorList) {
			if (evaluationFactor.getChildCount() > 0) {
				isHasParent = true;
			}
		}
		
		dataMap.put("evaluationFactorList", evaluationFactorList);
		dataMap.put("supplierList", supplierList);
		dataMap.put("supplierEvaluationFactorMap", supplierEvaluationFactorMap);
		dataMap.put("isHasParent", isHasParent);
		dataMap.put("PRODUCT_ROOT", FileConstants.PRODUCT_ROOT);
	}

	/**
	 * @Description: 过去供应商评审指标
	 * @author chenxw
	 * @param checkedSupplierList 
	 * @return 
	 * @throws Exception 
	 */
	private Map<String, EvaluationFactor> getSupplierEvaluationFactorList(List<Supplier> checkedSupplierList) throws Exception {
		Map<String, EvaluationFactor> map = new HashMap<String, EvaluationFactor>();
		for (Supplier supplier : checkedSupplierList) {
			// 获得投标人投标文件下的指标路径
			String supplierFactorXmlPath = FileConstants.getBidFilePath(tenderProjectEntity.getId(), supplier.getId()) + FileConstants.PSZB_XML;
			// 获得投标文件的指标Document
			Document factorDoc = XmlUtils.getXmlDocument(supplierFactorXmlPath);
			if(null == factorDoc){
				continue;
			}
			List<EvaluationFactor> evaluationFactorList = ParsingXmlUtils.parsingSupplierEvaluationFactorXML(factorDoc, tenderProjectEntity, evaluationStep);
			for (EvaluationFactor evaluationFactor : evaluationFactorList) {
				String key = supplier.getOrgCode() +"-" + evaluationFactor.getFactorCode();
				map.put(key, evaluationFactor);
			}
		}
		return map;
	}
	
	/**
	 * @Description: 生成按钮
	 * @author chenxw
	 * @param composite
	 * @param buttonName
	 * @return 
	 */
	private EButton newEButton(Composite composite, String buttonName) {
		EButton button = new EButton(composite, SWT.NONE);
		button.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		button.setBackground(ColorConstants.SKIN_BG);
		button.setForeground(ColorConstants.SKIN_FOREGROUND);
		button.setActiveBorderColor(ColorConstants.SKIN_ACTIVE_BG);
		button.setActiveBgColor(ColorConstants.SKIN_ACTIVE_BG);
		button.setActiveForeground(ColorConstants.SKIN_ACTIVE_FOREGROUND);
		button.setArcWidth(6);
		button.setText(buttonName);
		return button;
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	protected Point getInitialSize() {
		return new Point(800, 500);
	}
}
