package com.gx.obe.evaluation.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ELabel;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierOpeningResultService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 投标人废标
 * @author guoyr
 */
public class SupplierWasteBidDialog extends EDialog {
	
	private Text revokeReasonText;
	private Supplier supplier;
	private TenderProjectEntity tenderProjectEntity;
	private EButton revokeBtn;
	private CLabel supplierName;
	private boolean revokeSuccess = false;
	private SupplierOpeningResultService supplierOpeningResultService ;

	{
		LogUtils.logClass(SupplierWasteBidDialog.class);
	}

	/**
	 * @param parentShell
	 * @param tenderProjectEntity
	 * @param supplier
	 */
	public SupplierWasteBidDialog(Shell parentShell, TenderProjectEntity tenderProjectEntity, Supplier supplier) {
		super(parentShell);
		this.tenderProjectEntity = tenderProjectEntity;
		this.supplier = supplier;
		setTitle(com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, tenderProjectEntity.getTenderType())+"废标");
		supplierOpeningResultService = new SupplierOpeningResultService();
	}

	/** 
	 * @Description: 
	 * @param parent
	 * @return
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite top = (Composite) super.createDialogArea(parent);
		GridLayout gl_top = new GridLayout(1, false);
		gl_top.verticalSpacing = 0;
		gl_top.horizontalSpacing = 0;
		gl_top.marginHeight = 0;
		gl_top.marginWidth = 0;
		top.setLayout(gl_top);
		Composite container = new Composite(top, SWT.NONE);
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.marginHeight = 10;
		gl_container.marginWidth = 10;
		container.setLayout(gl_container);
		
		supplierName = new CLabel(container, SWT.NONE);
		supplierName.setFont(SWTResourceManager.getFont("宋体", 10, SWT.BOLD));
		supplierName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		supplierName.setText("北京国信创新科技股份有限公司");
		
		ELabel lblsbd = new ELabel(container, SWT.WRAP);
		lblsbd.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GRAY));
		lblsbd.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblsbd.setText("请输入废标原因");
		
		ESepator sep = new ESepator(top, SWT.SEPARATOR | SWT.HORIZONTAL);
		sep.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		sep.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		Composite composite = new Composite(top, SWT.NONE);
		FillLayout fl_composite = new FillLayout(SWT.HORIZONTAL);
		fl_composite.marginWidth = 2;
		composite.setLayout(fl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
		
		revokeReasonText = new Text(composite, SWT.WRAP | SWT.V_SCROLL);
		revokeReasonText.setTextLimit(200);
		
		ESepator label = new ESepator(top, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite_1 = new Composite(top, SWT.NONE);
		composite_1.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_composite_1 = new GridLayout(2, false);
		composite_1.setLayout(gl_composite_1);
		
		revokeBtn = new EButton(composite_1, SWT.NONE);
		revokeBtn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				importFileAction();
			}
		});
		GridData gd_revokeBtn = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		revokeBtn.setLayoutData(gd_revokeBtn);
		revokeBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/save.png"));
		revokeBtn.setText(Messages.Button_confirm);
		
		EButton cancel_btn = new EButton(composite_1, SWT.NONE);
		cancel_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		cancel_btn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				cancelAction();
			}
		});
		GridData gd_cancleBtn = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		cancel_btn.setLayoutData(gd_cancleBtn);
		cancel_btn.setText(Messages.Button_close);

		init();
		return top;
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr
	 */
	private void init(){
		if(null == supplier){
			revokeBtn.setEnabled(false);
		}
		supplierName.setText(supplier.getSupplierName());
	}

	/**
	 * @Description: 导入投标文件
	 * @author guoyr
	 */
	private void importFileAction(){
		if(revokeBtn.getEnabled()){
			if(revokeReasonText.getText().trim().length() > 0){
				if(MessageDialog.openConfirm(getShell(), "确认要将"+supplier.getSupplierName()+"的投标文件废标吗？")){
					
					SupplierOpeningResult supplierOpeningResult = supplierOpeningResultService.getSupplierOpeningResult(tenderProjectEntity.getId(), supplier.getId());
					boolean isNews = false;
					if(null == supplierOpeningResult){
						isNews = true;
						supplierOpeningResult = new SupplierOpeningResult();
					}
					supplierOpeningResult.setTenderId(tenderProjectEntity.getId());
					supplierOpeningResult.setSupplierId(supplier.getId());
					supplierOpeningResult.setRevokeStatus(SupplierEnum.REVOKE_YES);
					supplierOpeningResult.setRevokeReason(revokeReasonText.getText().trim());
					
					if(isNews){
						revokeSuccess = null != supplierOpeningResultService.saveOrUpdateNoEmptyReturn(supplierOpeningResult);
					}else {
						revokeSuccess = supplierOpeningResultService.updateAssignProperty(supplierOpeningResult, new String[]{"revokeStatus","revokeReason"});
					}
					if(revokeSuccess){
						getShell().close();
					}else {
						MessageDialog.openWarning(getShell(), ""+supplier.getSupplierName()+"废标失败！");
					}
				}
			}else {
				MessageDialog.openWarning(getShell(), "请输入"+supplier.getSupplierName()+"废标的原因！");
			}
		}
	}
	
	private void cancelAction(){
		revokeSuccess = false;
		this.getShell().close();
	}

	public boolean isRevokeSuccess() {
		return revokeSuccess;
	}

	public void setRevokeSuccess(boolean revokeSuccess) {
		this.revokeSuccess = revokeSuccess;
	}

	@Override
	protected Point getInitialSize() {
		return new Point(387, 253);
	}
}
