package com.gx.obe.evaluation.electionExpertLeader;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;

import com.gx.obe.ColorConstants;
import com.gx.obe.components.core.enumeration.ExpertEnum;
import com.gx.obe.listener.EvaluationRoomListener;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.web.entity.expert.Expert;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.expert.ExpertService;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 推荐专家组长
 * @author guoyr
 */
public class ElectionExpertLeaderComposite extends Composite implements EvaluationRoomListener {
	
	private ExpertService expertService = new ExpertService();
	
	private TenderProjectEntity tenderProject;
	private Expert curExpert;
	
	private CLabel message_label;
	private ScrolledComposite scrolledComposite;
	private Composite expert_list_composite;
	
	private Map<String, ExpertVoteComposite> expertVoteCompositeMap = new HashMap<String, ExpertVoteComposite>();
	
	private RefreshVotesThread refreshVotesThread;
	
	{
		LogUtils.logClass(ElectionExpertLeaderComposite.class);
	}
	
	/**
	 * @param parent
	 * @param tenderProject
	 * @param expert
	 */
	public ElectionExpertLeaderComposite(Composite parent, TenderProjectEntity tenderProject, Expert expert) {
		super(parent, SWT.NONE);
		addDisposeListener(new DisposeListener() {
			@Override
			public void widgetDisposed(DisposeEvent e) {
				if(null != refreshVotesThread) {
					refreshVotesThread.stop();
				}
				
			}
        });
		this.tenderProject = tenderProject;
		this.curExpert = expert;
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		
		message_label = new CLabel(this, SWT.NONE);
		message_label.setLeftMargin(8);
		message_label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		message_label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		message_label.setFont(SWTResourceManager.getFont("微软雅黑", 12, SWT.BOLD));
		message_label.setText("");
		
		scrolledComposite = new ScrolledComposite(this, SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		
		expert_list_composite = new Composite(scrolledComposite, SWT.NONE);
		expert_list_composite.setLayout(new RowLayout(SWT.HORIZONTAL));
		expert_list_composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		scrolledComposite.setContent(expert_list_composite);
		scrolledComposite.setMinSize(expert_list_composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		new ExpertVoteComposite(expert_list_composite, SWT.NONE);
		
		init();
	}
	
	/**
	 * @Description: 初始化界面数据
	 * @author guoyr
	 */
	private void init() {
		refreshExpertAction();
		refreshVotesThread = new RefreshVotesThread(tenderProject, new RefreshVotesThread.RefreshVotesListener() {
			public void refreshExpertVotes(final boolean isCanVote, final Map<String, Integer> votesMap, final int turnCount, final int noVoteCount) {
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						if (message_label.isDisposed()) {
							return;
						}
						for (String expertId : expertVoteCompositeMap.keySet()) {
							ExpertVoteComposite expertVoteComposite = expertVoteCompositeMap.get(expertId);
							expertVoteComposite.setVoteEnable(isCanVote);
							if (votesMap.containsKey(expertId)) {
								Integer voteCount = votesMap.get(expertId);
								expertVoteComposite.refreshVote(voteCount);
								if(isCanVote && turnCount > 1){
									if(voteCount < 1){
										expertVoteComposite.setVoteEnable(false);
									}
								}
							}
							expertVoteComposite.redraw();
						}
						if (isCanVote) {
							message_label.setText(MessageFormat.format(Messages.Tips_voteExpertLeaderRound, turnCount));
						} else if (noVoteCount != 0) {
							message_label.setText(Messages.Tips_youHaveVoted + MessageFormat.format(Messages.Tips_voteExpertLeaderCount, noVoteCount));
						}
					}
				});
			}
			
			public void existExpert(final String leaderName) {
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						if (message_label.isDisposed()) {
							return;
						}
						message_label.setText(MessageFormat.format(Messages.Tips_voteExpertLeaderResult, leaderName));
					}
				});
			}
		});
		refreshVotesThread.start();
	}
	
	/**
	 * @Description: 刷新专家推荐事件
	 * @author guoyr
	 */
	private void refreshExpertAction() {
		for (Control control : expert_list_composite.getChildren()) {
			control.dispose();
		}
		expertVoteCompositeMap.clear();
		List<Expert> expertList = expertService.getExpertList(tenderProject.getId(), ExpertEnum.ALL);
		if (null != expertList) {
			for (Expert expert : expertList) {
				initExpertVoteInfo(expert);
			}
		} 
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize(); // 取得当前屏幕属性
		double w = d.width * 0.8; // 获取屏幕宽度
		scrolledComposite.setMinSize(expert_list_composite.computeSize((int) (w - 50), SWT.DEFAULT));
	}
	
	/**
	 * @Description: 初始化专家投票信息
	 * @author chenxw
	 * @param expert
	 */
	private void initExpertVoteInfo(Expert expert) {
		ExpertVoteComposite composite = new ExpertVoteComposite(expert_list_composite, SWT.NONE);
		composite.initDate(tenderProject, expert, curExpert);
		composite.refreshVote(expert.getVotes());
		composite.setVoteEnable(false);
		composite.redraw();
		composite.addExpertVoteListener(new ExpertVoteComposite.ExpertVoteListener() {
			@Override
			public void expertVote() {
				if(!message_label.isDisposed()){
					for (String expertId : expertVoteCompositeMap.keySet()) {
						expertVoteCompositeMap.get(expertId).setVoteEnable(false);
						expertVoteCompositeMap.get(expertId).redraw();
					}
					message_label.setText(Messages.Tips_youHaveVoted);
				}
			}
		});
		expertVoteCompositeMap.put(expert.getId(), composite);
	}
	
	
	@Override
	public void dispose() {
		System.out.println("ElectionExpertLeaderComposite is being disposed");
		if(null != refreshVotesThread) {
			refreshVotesThread.stop();
		}
		super.dispose();
	}

	public void refresh(TenderProjectEntity tenderProject) {}
	
}
