package com.gx.obe.evaluation.electionExpertLeader;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gx.obe.components.core.Constants;
import com.gx.obe.web.entity.expert.Expert;

/**
 * @Description: 刷新投票
 * @author chenxw
 */
class RefreshVotes {
	
	private List<Expert> expertList;
	
	/**
	 * 当前专家是否可以投票
	 */
	boolean isCanVote;
	/**
	 * 当前投票轮次数
	 */
	int turnCount;
	/**
	 * 每一位专家的得票数量
	 */
	Map<String, Integer> votesMap = new HashMap<String, Integer>();
	
	/**
	 * 没有投票的专家数量
	 */
	int noVoteCount;
	
	int maxVateCount=0; //最大得票数
	/**
	 * 按专家得票数量由大到小排序
	 */
//	Set<Expert> votesSet = new TreeSet<Expert>(new Comparator<Expert>() {
//		public int compare(Expert o1, Expert o2) {
//			return Integer.compare(o2.getVotes(), o1.getVotes());
//		}
//	});
	
	/**
	 * 是否存在相同得票数量的专家
	 */
	private boolean isEqualVotes;
	
	void refreshData(List<Expert> expertList) {
		this.expertList = expertList;
		cleanDate();
		initVotesMap();
		initVotesSet();
	}
	
	/**
	 * @Description: 清空数据
	 * @author chenxw
	 */
	private void cleanDate() {
		isCanVote = false;
		turnCount = 1;
		maxVateCount = 0;
		votesMap.clear();
		noVoteCount = 0;
//		votesSet.clear();
		isEqualVotes = false;
	}
	
	/**
	 * @Description:
	 * @author chenxw
	 */
	private void initVotesMap() {
		//当前专家投票数
		int curVotesNum = 0;
		for (Expert expert : expertList) {
			if (Constants.USER.getId().equals(expert.getUserId())) {
				curVotesNum = expert.getVotesNum();
			}
			turnCount = Math.max(expert.getVotesNum(), turnCount);
			maxVateCount = Math.max(expert.getVotes(), maxVateCount);
			votesMap.put(expert.getId(), expert.getVotes());
		}
		isCanVote = curVotesNum < turnCount;
	}
	
	/**
	 * @Description:
	 * @author chenxw
	 */
	private void initVotesSet() {
		int maxVateCountNum = 0; //统计最大得票数量 一致的专家数量
		for (Expert expert : expertList) {
			if (turnCount > expert.getVotesNum()) {
				noVoteCount++;
			}
			if(maxVateCount == expert.getVotes()){
				maxVateCountNum ++;
			}
//			if (!votesSet.add(expert)) {
//				isEqualVotes = true;
//			}
		}
		// 如果当前轮次全部都投标完成，且最大投标数多于一个，则说明需要进入第二轮投标
		if (noVoteCount == 0 && maxVateCountNum != 1) {
			isEqualVotes = true;
			
			isCanVote = true;
			turnCount++;
		}
	}
	
	/**
	 * @Description: 是否产生组长
	 * @author chenxw
	 * @return
	 */
	boolean isGenerateLeader() {
		return maxVateCount > 0 && noVoteCount == 0 && !isEqualVotes;
	}
	
}