package com.gx.obe.evaluation.shell;

import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.CallBack;
import org.eclipse.swt.core.ImageResolver;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.LogUtils;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 用户登录缓存记录
 * @author guoyr
 */
public class CaListShell extends Shell {

	private Composite composite;
	private Image defaultAvatorImage;
	private ImageResolver imageResolver = new ImageResolver();
	private CallBack callBack;
	private Listener deactivateListener;
	
	{
		LogUtils.logClass(CaListShell.class);
	}
	
	/**
	 * Create the shell.
	 * @param display
	 */
	public CaListShell(Shell shell, CallBack callBack) {
		super(shell, SWT.NO_TRIM);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		this.callBack = callBack;
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginWidth = 1;
		gridLayout.marginHeight = 1;
		setLayout(gridLayout);
		
		composite = new Composite(this, SWT.NONE);
		composite.setBackgroundMode(SWT.INHERIT_FORCE);
		composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				removeListener(SWT.Deactivate, deactivateListener);
				imageResolver.disposite();
			}
		});
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.verticalSpacing = 0;
		gl_composite.horizontalSpacing = 0;
		gl_composite.marginHeight = 0;
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite userCmposite = new Composite(composite, SWT.NONE);
		GridLayout gl_userCmposite = new GridLayout(2, false);
		gl_userCmposite.verticalSpacing = 0;
		gl_userCmposite.horizontalSpacing = 0;
		gl_userCmposite.marginHeight = 0;
		gl_userCmposite.marginWidth = 0;
		userCmposite.setLayout(gl_userCmposite);
		GridData gd_userCmposite = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_userCmposite.heightHint = 70;
		userCmposite.setLayoutData(gd_userCmposite);
		
		CLabel userLael = new CLabel(userCmposite, SWT.NONE);
		userLael.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		userLael.setRightMargin(0);
		GridData gd_userLael = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_userLael.widthHint = 181;
		userLael.setLayoutData(gd_userLael);
		userLael.setText("New Label");
		
		CLabel deleteLabel = new CLabel(userCmposite, SWT.NONE);
		deleteLabel.setLeftMargin(0);
		deleteLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
		deleteLabel.setText("*");
		createContents();
		init();
	}
	
	private void init(){
		deactivateListener = new Listener(){
			public void handleEvent(Event arg0) {
				close();
			}
		};
		
		addListener(SWT.Deactivate, deactivateListener);
	}
	

	/**
	 * @Description: 列出历史用户登录记录
	 * @author guoyr
	 */
	public int listUserCache(){
		for(Control control : composite.getChildren()){
			control.dispose();	
		}
		defaultAvatorImage =  ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/male_48.png");
		Image deleteImage = ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/delete.png");
		
		int userCount = 0;
		File loginCacheFile = new File(FileConstants.SYS_USER_ROOT.concat("users"));
		if(null != loginCacheFile && loginCacheFile.exists()){
			File[] loginCaches = loginCacheFile.listFiles();
			if(null != loginCaches && loginCaches.length > 0){
				
				// 按登录时间进行排序
				List<File> cacheFileList = Arrays.asList(loginCaches);
				Collections.sort(cacheFileList, new SortComparator());
				for(final File cacheFile :  cacheFileList){
					if(userCount >= 8){
						break;
					}
					final Composite user_composite = new Composite(composite, SWT.NONE);
					user_composite.setBackgroundMode(SWT.INHERIT_FORCE);
					GridLayout gl_userCmposite = new GridLayout(2, false);
					gl_userCmposite.verticalSpacing = 0;
					gl_userCmposite.horizontalSpacing = 0;
					gl_userCmposite.marginHeight = 0;
					gl_userCmposite.marginWidth = 0;
					user_composite.setLayout(gl_userCmposite);
					GridData gd_userCmposite = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
					user_composite.setLayoutData(gd_userCmposite);
					
					File imageFile = new File(cacheFile.getAbsolutePath() + File.separator + "avatar.png");
					Image avatar = defaultAvatorImage;
					if(imageFile.exists()){
						avatar = imageResolver.loadImage(imageFile);
					}
					if(null == avatar){
						avatar = defaultAvatorImage;
					}
					final ImageButton user_cache = new ImageButton(user_composite, SWT.LEFT);
					user_cache.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
					user_cache.setForeground(SWTResourceManager.getColor(100, 100, 100));
					GridData gd_userLael = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
					gd_userLael.widthHint = 60;
					user_cache.setLayoutData(gd_userLael);
					user_cache.setSurroundColor(SWTResourceManager.getColor(SWT.COLOR_WHITE));
//					user_cache.setActiveSurroundColor(SWTResourceManager.getColor(SWT.COLOR_WHITE));
					user_cache.setActiveBgColor(ColorConstants.SELECT_BACKGROUND);
					user_cache.setLeftMargin(10);
//					user_cache.setSurroundMargin(2);
					user_cache.setGAP(20);
					user_cache.setText(cacheFile.getName());
					user_cache.addMouseTrackListener(new UserCacheMouseTrackListener(user_composite,user_cache, avatar));
					
					if(userCount++ == 0){
						user_cache.notifyListeners(SWT.MouseEnter, null);
					}else {
						user_cache.setImage(avatar, 32, 32);
					}
					user_cache.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseUp(MouseEvent e) {
							if(null != callBack){
								close();
								callBack.callBackString(user_cache.getText());
							}
						}
					});
					final CLabel delete = new CLabel(user_composite, SWT.NONE);
					delete.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
					delete.setLeftMargin(0);
					delete.setImage(deleteImage);
					delete.setToolTipText(Messages.Button_delete);
					
					
					delete.addMouseTrackListener(new UserCacheMouseTrackListener(user_composite,user_cache, avatar));
					delete.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseUp(MouseEvent e) {
							user_composite.dispose();
							composite.layout();
							int height = composite.getShell().computeSize(composite.getShell().getSize().x, SWT.DEFAULT).y + 20;
							FileUtils.delFolder(cacheFile.getAbsolutePath());
							composite.getShell().setSize(composite.getShell().getSize().x, Math.max(height, 50));
						}
					});
					composite.layout();
				}
			}
		}
		return userCount;
	}
	/**
	 * @Description: 鼠标移动至菜单上的监听
	 * @author guoyr
	 * @param composite
	 * @param actionLabel
	 */
	class UserCacheMouseTrackListener extends MouseTrackAdapter{
		private Composite actionComposite;
		private ImageButton actionLabel;
		private Image image;
		public UserCacheMouseTrackListener(Composite actionComposite, ImageButton actionLabel, Image image){
			this.actionComposite = actionComposite;
			this.actionLabel = actionLabel;
			this.image = image;
		}
		public void mouseEnter(MouseEvent e) {
			actionComposite.setBackground(ColorConstants.SELECT_BACKGROUND);
			actionLabel.setImage(image, 40, 40);
			composite.layout();
		}
		public void mouseExit(MouseEvent e) {
			actionComposite.setBackground(null);
			actionLabel.setImage(image, 32, 32);
			composite.layout();
		}
	}
	
	/**
	 * @Description: 按登录时间进行排序
	 * @author guoyr
	 */
	private class SortComparator implements Comparator<File> {
		public int compare(File file1, File file2) {
			if(file1.lastModified() > file2.lastModified()){
				return -1;
			}else {
				return 1;
			}
		}
	}

	
	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText("");
		setSize(290, 100);

	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
