package com.gx.obe.evaluation.shell;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.tools.ftp.FtpDownLoadListener;
import com.gx.tools.ftp.FtpDownLoadThread;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class DownloadItemComposite extends Composite implements IDownloadItem{

	private Label supplier_speedLabel;
	private Label supplierName_label;
	private ProgressBar supplier_progressBar;
	private FtpDownLoadListener ftpDownLoadListener;
	private FtpDownLoadThread ftpDownLoadThread;
	private ImageButton button;
//	private Label message_label;
	private boolean downloading = false;
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public DownloadItemComposite(Composite parent, int style) {
		super(parent, style);
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent arg0) {
				stop();
			}
		});

		GridLayout gl_composite_2 = new GridLayout(2, false);
		gl_composite_2.verticalSpacing = 0;
		gl_composite_2.horizontalSpacing = 0;
		gl_composite_2.marginWidth = 0;
		gl_composite_2.marginHeight = 5;
		setLayout(gl_composite_2);
		
		Composite composite = new Composite(this, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_composite = new GridLayout(3, false);
		gl_composite.horizontalSpacing = 0;
		gl_composite.marginHeight = 0;
		gl_composite.verticalSpacing = 0;
		composite.setLayout(gl_composite);
		
		button = new ImageButton(composite, SWT.CHECK);
		button.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				download();
			}
		});
		button.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 2));
		button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/startDownload.png"));
		button.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		button.setActiveSurroundColor(SWTResourceManager.getColor(36, 118, 210));
		button.setActiveSurroundColor(SWTResourceManager.getColor(255, 153, 0));
//		button.setSurroundColor(SWTResourceManager.getColor(36, 118, 210));
		button.setSurroundMargin(0);
//		button.setFillSurround(false);
		button.setCursor(SWTResourceManager.getCursor(SWT.CURSOR_HAND));
		button.setText("");
		
		
		supplierName_label = new Label(composite, SWT.WRAP);
		supplierName_label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		supplierName_label.setForeground(ColorConstants.FONT_BLACK_COLOR);
		GridData gd_messageLabel = new GridData(SWT.FILL, SWT.CENTER, true, true, 2, 1);
		gd_messageLabel.widthHint = 250;
		supplierName_label.setLayoutData(gd_messageLabel);
		
		supplier_progressBar = new ProgressBar(composite, SWT.NONE);
		GridData gd_progressBar = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_progressBar.heightHint = 10;
		gd_progressBar.widthHint = 205;
		supplier_progressBar.setLayoutData(gd_progressBar);
		
		supplier_speedLabel = new Label(composite, SWT.WRAP);
		GridData gd_speedLabel = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		gd_speedLabel.widthHint = 90;
		supplier_speedLabel.setLayoutData(gd_speedLabel);
		supplier_speedLabel.setForeground(SWTResourceManager.getColor(100, 100, 100));
		supplier_speedLabel.setText("0.00B/S");
		
//		message_label = new Label(composite, SWT.NONE);
//		message_label.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
//		message_label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
//		message_label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
//		message_label.setText("");
		
		init();
	}

	
	private void init(){
		ftpDownLoadListener = new FtpDownLoadListener(){
			public void info(final String info) {
//				Display.getDefault().syncExec(new Runnable(){
//					public void run(){
//						if(!message_label.isDisposed()){
//							message_label.setText(info);
//						}
//					}
//				});
			}
			public void error(final String error) {
//				Display.getDefault().syncExec(new Runnable(){
//					public void run(){
//						if(!message_label.isDisposed()){
//							message_label.setText(error);
//						}
//					}
//				});
			}
		};
	}
	
	public void refresh(String title, String hostname, String username, String password, int port, String remotedir, String downLoadPath){
		supplierName_label.setText(title);
		ftpDownLoadThread = new FtpDownLoadThread(hostname, port, username, password, remotedir, downLoadPath, ftpDownLoadListener);
	}
	
	private void download(){
		if(downloading){
			pause();
		}else {
			start();
		}
	}
	
	public void start(){
		if(null != ftpDownLoadThread){
//			supplier_progressBar.setSelection(0);
			downloading = true;
			button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/pauseDownload.png"));
			ftpDownLoadThread.start();
			new Thread(new Runnable(){
				public void run(){
					Runnable infoRunnable = new Runnable(){
						public void run(){
							if(!supplier_progressBar.isDisposed()){
								supplier_speedLabel.setText(ftpDownLoadListener.getSpeed());
								supplier_progressBar.setSelection((int)ftpDownLoadListener.getProgress());
							}
						}
					};
					while(true){
						Display.getDefault().syncExec(infoRunnable);
						if(ftpDownLoadListener.isFinish()){
							downloading = false;
							Display.getDefault().syncExec(new Runnable(){
								public void run(){
									if(!button.isDisposed()){
										button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/startDownload.png"));
									}
								}
							});
							break;
						}
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e) {
						}
					}
				}
			}).start();
		}
	}
	
	public void pause(){
		if(null != ftpDownLoadThread){
			Display.getDefault().syncExec(new Runnable(){
				public void run(){
					if(!button.isDisposed()){
						button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/startDownload.png"));
					}
				}
			});
			downloading = false;
			ftpDownLoadThread.pause();
		}
	}
	public void stop(){
		if(null != ftpDownLoadThread){
			Display.getDefault().syncExec(new Runnable(){
				public void run(){
					if(!button.isDisposed()){
						button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/startDownload.png"));
					}
				}
			});
			downloading = false;
			ftpDownLoadThread.stop();
		}
	}
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
