package com.gx.obe.evaluation.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.RemoteFile;

/**
 * @Description: Ftp下载投标文件
 * @author chenxw
 */
public class FtpBidFileDownloadThread implements Runnable{

	private boolean isDeleteSourceFile = false;
	private FTPClient ftpClient = new FTPClient();
	private static String DEAFULT_REMOTE_CHARSET = "GBK";
	private static int DEAFULT_REMOTE_PORT = 21;// 默认端口
	private Logger LOG = Logger.getLogger(FtpBidFileDownloadThread.class);
	private List<File> deleteSourceFileList = new ArrayList<File>();
	private List<RemoteFile> remoteFileList;
	private FTPCallBack callBack;
	private boolean downLoadFlag = false;
	private double lRemoteSize = 0;
	private double downloadSize = 0;

	public FtpBidFileDownloadThread(List<RemoteFile> remoteFileList, FTPCallBack ftpCallBack) {
		this.remoteFileList = remoteFileList;
		this.callBack = ftpCallBack;
	}
	public void run() {
		try {
			if (connection()) {
				for (final RemoteFile remoteFile : remoteFileList) {
					
					download(ftpClient, remoteFile);
				}
			} else {
			}
		} catch (Exception e) {
			LOG.error("FTP下载失败！", e);
		} finally {
			try {
				logOut();
			} catch (IOException e) {
				LOG.error("FTP注销失败！", e);
			}
			if (isDeleteSourceFile) {
				deleteDownloadFile();
			}
		}
	}

	/**
	 * @Description: 注销Ftp
	 * @author guoyr
	 * @throws IOException
	 */
	private void logOut() throws IOException {
		if (null != ftpClient) {
			// FTP退出
			ftpClient.logout();
			// 断开FTP连接
			if (ftpClient.isConnected()) {
				ftpClient.disconnect();
			}
		}
	}

	/**
	 * @Description: 从FTP服务器上下载文件,支持断点续传，上传百分比汇报
	 * @author guoyr
	 * @param ftpClient
	 * @param remoteFile远程文件
	 * @param remoteFilePath 远程文件路径
	 * @param localFilePath 远程文件路径
	 * @return
	 */
	private boolean download(FTPClient ftpClient, RemoteFile remoteFile) {
		boolean download = false;
		FileOutputStream out = null;
		InputStream in = null;
		System.out.println();
		// 本地存在文件，进行断点下载
		String bidFileURL = FileConstants.BID_FILE_PATH;
	
		boolean isDdxc = false;
		callBack.callBackString("正在下载更新包，请稍后......");
		try {
			lRemoteSize = remoteFile.getSize();
			
			File localFile = new File(bidFileURL + remoteFile.getName());
			if (localFile.exists()) {
				long localSize = localFile.length();
				downloadSize = localSize;
				// 判断本地文件大小是否大于远程文件大小
				if (localSize == lRemoteSize) {
					LOG.debug(localFile.getName() + "已存在！");
					download = true;
					callBack.callBackString("更新包下载成功！");
					callBack.selectProgressBar(100);
					return download;
				} else {
					// 进行断点续传，并记录状态
					isDdxc = true;
					ftpClient.setRestartOffset(localSize);
				}
			} else {
				if (!localFile.getParentFile().exists()) {
					localFile.getParentFile().mkdirs();
				}
			}
			deleteSourceFileList.add(localFile);
			new Thread(new Runnable(){
				public void run(){
					while(!downLoadFlag){
						String value = ((downloadSize  * 100 /lRemoteSize)) + "";
						if(value.indexOf(".") != -1){
							value = value.substring(0 , value.indexOf("."));
						}
						callBack.selectProgressBar(Integer.parseInt(value));
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			}).start();
			
			
			byte[] bytes = new byte[1024];
			in = ftpClient.retrieveFileStream(remoteFile.getName());
			if (null != in) {
				out = new FileOutputStream(localFile, isDdxc);
				int c;
				while ((c = in.read(bytes)) != -1) {
					out.write(bytes, 0, c);
					downloadSize = downloadSize + c;
				}
				download = ftpClient.completePendingCommand();
				downLoadFlag = true;
			}
			callBack.callBackString("更新包下载成功！");
		} catch (IOException e) {
			download = false;
			LOG.error(remoteFile.getPath() + "下载失败！", e);
		}  finally {
			if (null != in) {
				try {
					in.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
			}
			if (null != out) {
				try {
					out.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
			}
			
			callBack.startUpdate(bidFileURL + remoteFile.getName());
		}
		return download;
	}

	/**
	 * @Description: 连接FTP服务器
	 * @author chenxw
	 * @return
	 */
	private boolean connection() {
		boolean connection = false;
		String username = "anonymous";
		String password = null;
		int port = 21;
		String hostname = PropertiesUtils.getProperty(ConfigConstants.FTP_IP, "");
		try {
			port = Integer.parseInt(PropertiesUtils.getProperty(ConfigConstants.FTP_PORT, "21"));
		} catch (NumberFormatException e) {
		}
		
		username = PropertiesUtils.getProperty(ConfigConstants.FTP_USERNAME, username);
		password = PropertiesUtils.getProperty(ConfigConstants.FTP_PASSWORD, password);
//		if (hostname.equals(SystemTool.getIp())) {
//			return false;
//		}
		try {
			// 登录FTP服务器
			try {
				// 设置超时时间
				ftpClient.setDataTimeout(7200);
				// 设置默认编码
				ftpClient.setControlEncoding(DEAFULT_REMOTE_CHARSET);
				// 设置默认端口
				ftpClient.setDefaultPort(DEAFULT_REMOTE_PORT);
				// 设置是否显示隐藏文件
				ftpClient.setListHiddenFiles(false);
				// 连接ftp服务器
				if (port > 0) {
					ftpClient.connect(hostname, port);
				} else {
					ftpClient.connect(hostname);
				}
				connection = true;
			} catch (ConnectException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (SocketException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (IOException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			}
			// 连接成功后登录FTP服务器
			if (connection) {
				// 得到连接的返回编码
				int reply = ftpClient.getReplyCode();
				if (!FTPReply.isPositiveCompletion(reply)) {
					ftpClient.disconnect();
				}
				// 登录失败权限验证失败
				if (!ftpClient.login(username, password)) {
					ftpClient.quit();
					ftpClient.disconnect();
					LOG.error("连接FTP服务器用户或者密码失败：" + ftpClient.getReplyString());
					connection = false;
				}
				if (LOG.isDebugEnabled()) {
					LOG.info("成功登录FTP服务器：" + hostname + " 端口：" + port);
				}
				ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
				// 设置被动模式
				ftpClient.enterLocalPassiveMode();
				// 设置以二进制方式传输
				ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
			} else {
				connection = false;
			}
		} catch (IOException e) {
			connection = false;
			LOG.error("登录FTP服务器失败：" + hostname + " 端口：" + port);
		}
		return connection;
	}

	public void start() {
		new Thread(this).start();
	}

	public void initDownload() {
		ftpClient = new FTPClient();
		isDeleteSourceFile = false;
		deleteSourceFileList.clear();
	}

	public void setDeleteDownloadFile(boolean isDelete) {
		this.isDeleteSourceFile = isDelete;
	}

	public void deleteDownloadFile() {
		for (File deleteSourceFile : deleteSourceFileList) {
			if (null != deleteSourceFile && deleteSourceFile.exists()) {
				deleteSourceFile.delete();
			}
		}
	}

	public Runnable getRunnable() {
		return this;
	}
	public static interface FTPCallBack {
		public void callBackString(String returnMessage);
		public void startUpdate(String updateFilePath);
		public void selectProgressBar(int value);
	}
}
