package com.gx.obe.evaluation.thread;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.ConnectException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import com.gx.obe.common.widget.thread.MyThread;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.RemoteFile;
import com.gx.security.SystemTool;

/** 
 * @Description: 获取FTP服务器文件信息
 * @author chenxw 
 */
public class FtpCheckUpdateFileThread extends MyThread {

	private CheckCallBack checkCallBack;
	private Logger LOG = Logger.getLogger(FtpCheckUpdateFileThread.class);
	private FTPClient ftpClient = new FTPClient();
//	private String remotedir;
	private long totalSize = 0;
	public FtpCheckUpdateFileThread(CheckCallBack checkCallBack) {
		this.checkCallBack = checkCallBack;
	}

	public void start() {
		thread = new Thread(this);
		thread.start();
	}

	public void run() {
		List<RemoteFile> remoteFileList = null;
		try {
			if (!connection()) {
				return;
			}
			remoteFileList = getDownLoadFTPFiles(ftpClient);
		} catch (Exception e) {
			LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
		} finally {
			if (ftpClient.isConnected()) {
				try {
					ftpClient.disconnect();
				} catch (IOException e) {
				}
			}
			checkCallBack.bidFile(totalSize, remoteFileList);
		}
	}

	/**
	 * @Description: 连接FTTP服务器
	 * @author chenxw
	 * @return 
	 */
	private boolean connection() {
		String hostname = PropertiesUtils.getProperty(ConfigConstants.FTP_IP, "");
		String username = PropertiesUtils.getProperty(ConfigConstants.FTP_USERNAME, null);
		String password = PropertiesUtils.getProperty(ConfigConstants.FTP_PASSWORD, null);
		int port = 21;
		try {
			port = Integer.parseInt(PropertiesUtils.getProperty(ConfigConstants.FTP_PORT, "21"));
		} catch (NumberFormatException e) {
		}

		if (null == hostname || hostname.length() == 0) {
			LOG.error("未设置FTP服务器IP");
			return false;
		}

		hostname = hostname.trim();
		String localMachine = "localhost,127.0.0.1," + SystemTool.getIp();
		
		if(localMachine.contains(hostname)){
			return false;
		}
		boolean connection = false;
		try {
			//登录FTP服务器
			try {
				ftpClient.setConnectTimeout(3000);
				if (port > 0) {
					ftpClient.connect(hostname, port);
				} else {
					ftpClient.connect(hostname);
				}
				connection = true;
			} catch (ConnectException e) {
				LOG.error("FTP服务器连接失败：", e);
			} catch (SocketException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (IOException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			}
			// 连接成功后登录FTP服务器
			if (connection) {
				//得到连接的返回编码
				if (!FTPReply.isPositiveCompletion(ftpClient.getReplyCode())) {
					ftpClient.disconnect();
				}
				//登录失败、权限验证失败
				if (ftpClient.login(username, password)) {
					//设置超时时间  
					ftpClient.setDataTimeout(7200);
					//设置默认编码
					ftpClient.setControlEncoding("GBK");
					//设置默认端口
					ftpClient.setDefaultPort(21);
					//设置是否显示隐藏文件
					ftpClient.setListHiddenFiles(false);
					// 设置被动模式
					ftpClient.enterLocalPassiveMode();
					// 设置以二进制方式传输
					ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
					//设置ftp工作目录
//					ftpClient.changeWorkingDirectory(remotedir);

					LOG.info("成功登录FTP服务器：" + hostname + " 端口：" + port);
				} else {
					connection = false;
					//退出
					ftpClient.quit();
					//断开
					ftpClient.disconnect();
					LOG.error("连接FTP服务器用户名或密码错误：" + ftpClient.getReplyString());
				}
			} 
		} catch (IOException e) {
			connection = false;
			LOG.error("登录FTP服务器失败：" + hostname + " 端口：" + port);
		}
		return connection;
	}

	/**
	 * @Description: 获取FTP服务器上要下载的文件
	 * @author chenxw
	 * @param ftp
	 * @param remoteDir
	 * @return 
	 */
	private List<RemoteFile> getDownLoadFTPFiles(FTPClient ftp) {
		String oldVersion = PropertiesUtils.getProperty(ConfigConstants.VERSION,"V4.0");
		List<RemoteFile> downLoadFTPFiles = new ArrayList<RemoteFile>();
		try {
			for (FTPFile ftpfile : ftp.listFiles()) {
				if (!ftpfile.getName().contains("update")) {
					continue;
				}
				String fileName = ftpfile.getName();
				if(fileName.indexOf("_") != -1 && fileName.indexOf(".") != -1){
					String versionMessage = fileName.substring(fileName.indexOf("_")+1);
					String newVersion = versionMessage.substring(0, versionMessage.lastIndexOf("."));
					if(compareVersonString(oldVersion, newVersion)){
//						String remotePath = remoteDir;
						RemoteFile file = new RemoteFile();
						file.setPath(fileName);
						file.setName(ftpfile.getName());
						if (ftpfile.isFile() && ftpfile.getSize() > 0) {
							totalSize += ftpfile.getSize();
							file.setSize(ftpfile.getSize());
							file.setIsDirectory(false);
							downLoadFTPFiles.add(file);
						} else if (ftpfile.isDirectory()) {
							file.setIsDirectory(true);
							downLoadFTPFiles.add(file);
						}
					}
				}
				
			}
		} catch (IOException e) {
			LOG.error("获取ftp服务器文件失败！", e);
		}
		return downLoadFTPFiles;
	}

	public static interface CheckCallBack {
		public void bidFile(long totalSize, List<RemoteFile> downLoadFTPFiles);
//		public void callBackConnectOut(String message);
	}
	
	/**
	 * 比较新旧版本
	 * @param oldVersion
	 * @param newVersion
	 * @return
	 */
	private boolean compareVersonString(String oldVersion, String newVersion){
		String thisOldVersion = oldVersion.substring(oldVersion.indexOf("V") + 1);
		if(thisOldVersion.indexOf("_") > 0){
			thisOldVersion = thisOldVersion.substring(0, thisOldVersion.indexOf("_"));
		}
		String thisNewVersion = newVersion.substring(oldVersion.indexOf("V") + 1);
		if(thisNewVersion.indexOf("_") > 0){
			thisNewVersion = thisNewVersion.substring(0, thisNewVersion.indexOf("_"));
		}
		String[] oldVersionNum = thisOldVersion.split("\\.");
		String[] newVersionNum = thisNewVersion.split("\\.");
		if(newVersionNum.length > oldVersionNum.length){
			for(int i = 0; i < newVersionNum.length; i++){
				String newVStr = newVersionNum[i];
				int newV = Integer.parseInt(newVStr);
				if(oldVersionNum.length > i){
					String oldVStr = oldVersionNum[i];
					int oldV = Integer.parseInt(oldVStr);
					if(newV == oldV){
						continue;
					}else if(newV > oldV){
						return true;
					}else {
						return false;
					}
				}else {
					return true;
				}
			}
		}else {
			for(int i = 0; i < oldVersionNum.length; i++){
				String oldVStr = oldVersionNum[i];
//				Long oldV =  Long.getLong(oldVStr);
				if(newVersionNum.length > i){
					String newVStr = newVersionNum[i];
//					Long newV = Long.getLong(newVStr);
					if((new BigDecimal(newVStr)).compareTo(new BigDecimal(oldVStr)) == 0){
						continue;
					}else if((new BigDecimal(newVStr)).compareTo(new BigDecimal(oldVStr)) > 0){
						return true;
					}else {
						return false;
					}
				}
			}
			
		}
		return false;
	}
}
